@@ -32,7 +32,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <syslog.h>
 #include <sys/file.h>
 #include <sys/mman.h>
 #include <sys/types.h>
@@ -45,11 +44,6 @@
 #include <malloc.h>
 #endif
 
-/* FIXME Use dm library */
-#define	dbg_malloc(x...)	malloc(x)
-#define	dbg_strdup(x...)	strdup(x)
-#define	dbg_free(x...)		free(x)
-
 /* List (un)link macros. */
 #define	LINK(x, head)		list_add(head, &(x)->list)
 #define	LINK_DSO(dso)		LINK(dso, &dso_registry)
@@ -132,7 +126,8 @@ struct thread_status {
 	struct dso_data *dso_data;/* DSO this thread accesses. */
 	
 	char *device_path;	/* Mapped device path. */
-	int event_nr;           /* event number */
+	uint32_t event_nr;           /* event number */
+	int processing;         /* Set when event is being processed */
 	enum dm_event_type events;	/* bitfield for event filter. */
 	enum dm_event_type current_events;/* bitfield for occured events. */
 	enum dm_event_type processed_events;/* bitfield for processed events. */
@@ -141,6 +136,7 @@ struct thread_status {
 	struct list timeout_list;
 };
 static LIST_INIT(thread_registry);
+static LIST_INIT(thread_registry_unused);
 
 static int timeout_running;
 static LIST_INIT(timeout_registry);
@@ -151,12 +147,12 @@ static pthread_cond_t timeout_cond = PTHREAD_COND_INITIALIZER;
 static struct thread_status *alloc_thread_status(struct message_data *data,
 						 struct dso_data *dso_data)
 {
-	struct thread_status *ret = (typeof(ret)) dbg_malloc(sizeof(*ret));
+	struct thread_status *ret = (typeof(ret)) dm_malloc(sizeof(*ret));
 
 	if (ret) {
 		if (!memset(ret, 0, sizeof(*ret)) ||
-		    !(ret->device_path = dbg_strdup(data->device_path))) {
-			dbg_free(ret);
+		    !(ret->device_path = dm_strdup(data->device_path))) {
+			dm_free(ret);
 			ret = NULL;
 		} else {
 			ret->dso_data = dso_data;
@@ -171,19 +167,19 @@ static struct thread_status *alloc_thread_status(struct message_data *data,
 
 static void free_thread_status(struct thread_status *thread)
 {
-	dbg_free(thread->device_path);
-	dbg_free(thread);
+	dm_free(thread->device_path);
+	dm_free(thread);
 }
 
 /* Allocate/free DSO data. */
 static struct dso_data *alloc_dso_data(struct message_data *data)
 {
-	struct dso_data *ret = (typeof(ret)) dbg_malloc(sizeof(*ret));
+	struct dso_data *ret = (typeof(ret)) dm_malloc(sizeof(*ret));
 
 	if (ret) {
 		if (!memset(ret, 0, sizeof(*ret)) ||
-		    !(ret->dso_name = dbg_strdup(data->dso_name))) {
-			dbg_free(ret);
+		    !(ret->dso_name = dm_strdup(data->dso_name))) {
+			dm_free(ret);
 			ret = NULL;
 		}
 	}
@@ -193,8 +189,8 @@ static struct dso_data *alloc_dso_data(struct message_data *data)
 
 static void free_dso_data(struct dso_data *data)
 {
-	dbg_free(data->dso_name);
-	dbg_free(data);
+	dm_free(data->dso_name);
+	dm_free(data);
 }
 
 /* FIXME: Factor out. */
@@ -220,11 +216,11 @@ static int fetch_string(char **ptr, char **src)
 	if ((p = strchr(*src, delimiter)))
 		*p = 0;
 
-	if ((*ptr = dbg_strdup(*src))) {
+	if ((*ptr = dm_strdup(*src))) {
 		if ((len = strlen(*ptr)))
 			*src += len;
 		else {
-			dbg_free(*ptr);
+			dm_free(*ptr);
 			*ptr = NULL;
 		}
 
@@ -242,10 +238,10 @@ static int fetch_string(char **ptr, char **src)
 static void free_message(struct message_data *message_data)
 {
 	if (message_data->dso_name)
-		dbg_free(message_data->dso_name);
+		dm_free(message_data->dso_name);
 
 	if (message_data->device_path)
-		dbg_free(message_data->device_path);
+		dm_free(message_data->device_path);
 }
 
 /* Parse a register message from the client. */
@@ -268,12 +264,12 @@ static int parse_message(struct message_data *message_data)
 			 * Free string representaion of events.
 			 * Not needed an more.
 			 */
-			dbg_free(message_data->events.str);
+			dm_free(message_data->events.str);
 			message_data->events.field = i;
 		}
 		if (message_data->timeout.str) {
 			uint32_t secs = atoi(message_data->timeout.str);
-			dbg_free(message_data->timeout.str);
+			dm_free(message_data->timeout.str);
 			message_data->timeout.secs = secs ? secs :
 							    DM_EVENT_DEFAULT_TIMEOUT;
 		}
@@ -304,10 +300,10 @@ static int storepid(int lf)
 	if ((len = snprintf(pid, sizeof(pid), "%u\n", getpid())) < 0)
 		return 0;
 
-	if (len > sizeof(pid))
-		len = sizeof(pid);
+	if (len > (int) sizeof(pid))
+		len = (int) sizeof(pid);
 
-	if (write(lf, pid, len) != len)
+	if (write(lf, pid, (size_t) len) != len)
 		return 0;
 
 	fsync(lf);
@@ -315,19 +311,29 @@ static int storepid(int lf)
 	return 1;
 }
 
-
+/* FIXME This is unreliable: should use DM_DEVICE_INFO ioctl instead. */
 /* Check, if a device exists. */
 static int device_exists(char *device)
 {
 	struct stat st_buf;
+	char path2[PATH_MAX];
 
-	return !stat(device, &st_buf) && S_ISBLK(st_buf.st_mode);
+	if (!device)
+		return 0;
+
+	if (device[0] == '/') /* absolute path */
+		return !stat(device, &st_buf) && S_ISBLK(st_buf.st_mode);
+
+	if (PATH_MAX <= snprintf(path2, PATH_MAX, "%s/%s", dm_dir(), device))
+		return 0;
+
+	return !stat(path2, &st_buf) && S_ISBLK(st_buf.st_mode);
 }
 
 /*
  * Find an existing thread for a device.
  *
- * Mutex must be hold when calling this.
+ * Mutex must be held when calling this.
  */
 static struct thread_status *lookup_thread_status(struct message_data *data)
 {
@@ -578,6 +584,8 @@ static void monitor_unregister(void *arg)
 }
 
 /* Device monitoring thread. */
+static void *monitor_thread(void *arg)
+    __attribute((noreturn));
 static void *monitor_thread(void *arg)
 {
 	struct thread_status *thread = arg;
@@ -585,7 +593,7 @@ static void *monitor_thread(void *arg)
 	pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
 	pthread_cleanup_push(monitor_unregister, thread);
 
-	/* Wait for do_process_reques() to finish its task. */
+	/* Wait for do_process_request() to finish its task. */
 	lock_mutex();
 	unlock_mutex();
 
@@ -593,6 +601,11 @@ static void *monitor_thread(void *arg)
 	while (1) {
 		thread->current_events = 0;
 
+		/*
+		 * FIXME: if unrecoverable error (ENODEV) happens,
+		 * we loop indefinitely.  event_wait should return
+		 * more than 0/1.
+		 */
 		if (!event_wait(thread))
 			continue;
 
@@ -603,12 +616,22 @@ static void *monitor_thread(void *arg)
 		 * the device got registered for those events AND
 		 * those events haven't been processed yet, call
 		 * the DSO's process_event() handler.
+		 *
+		 * FIXME: when does processed_events get cleared?  What if
+		 * the same type of event happens later... after the first
+		 * was handled properly?
 		 */
 		if (thread->events &
 		    thread->current_events &
 		    ~thread->processed_events) {
+			lock_mutex();
+			thread->processing = 1;
+			unlock_mutex();
 			do_process_event(thread);
 			thread->processed_events |= thread->current_events;
+			lock_mutex();
+			thread->processing = 0;
+			unlock_mutex();
 		}
 	}
 
@@ -689,41 +712,27 @@ static int lookup_symbols(void *dl, struct dso_data *data)
 			     "unregister_device");
 }
 
-/* Create a DSO file name based on its name. */
-static char *create_dso_file_name(char *dso_name)
-{
-	char *ret;
-	static char prefix[] = "libdmeventd";
-	static char suffix[] = ".so";
-
-	if ((ret = dbg_malloc(strlen(prefix) +
-			      strlen(dso_name) +
-			      strlen(suffix) + 1)))
-		sprintf(ret, "%s%s%s", prefix, dso_name, suffix);
-
-	return ret;
-}
-
 /* Load an application specific DSO. */
 static struct dso_data *load_dso(struct message_data *data)
 {
 	void *dl;
 	struct dso_data *ret = NULL;
-	char *dso_file;
 
-	if (!(dso_file = create_dso_file_name(data->dso_name)))
-		return NULL;
-
-	if (!(dl = dlopen(dso_file, RTLD_NOW))){
+	if (!(dl = dlopen(data->dso_name, RTLD_NOW))){
 		log_error("%s\n", dlerror());
-		goto free_dso_file;
+		return NULL;
 	}
 
-	if (!(ret = alloc_dso_data(data)))
-		goto close;
+	if (!(ret = alloc_dso_data(data))) {
+		dlclose(dl);
+		return NULL;
+	}
 
-	if (!(lookup_symbols(dl, ret)))
-		goto free_all;
+	if (!(lookup_symbols(dl, ret))) {
+		free_dso_data(ret);
+		dlclose(dl);
+		return NULL;
+	}
 
 	/*
 	 * Keep handle to close the library once
@@ -736,17 +745,6 @@ static struct dso_data *load_dso(struct message_data *data)
 	LINK_DSO(ret);
 	unlock_mutex();
 
-	goto free_dso_file;
-
-   free_all:
-	free_dso_data(ret);
-
-   close:
-	dlclose(dl);
-
-   free_dso_file:
-	dbg_free(dso_file);
-
 	return ret;
 }
 
@@ -875,27 +873,11 @@ static int unregister_for_event(struct message_data *message_data)
 	 * In case there's no events to monitor on this device ->
 	 * unlink and terminate its monitoring thread.
 	 */
-	if (!thread->events)
-		UNLINK_THREAD(thread);
-
-	unlock_mutex();
-
 	if (!thread->events) {
-		/* turn codes negative */
-		if ((ret = -terminate_thread(thread)))
-			stack;
-		else {
-			pthread_join(thread->thread, NULL);
-			free_thread_status(thread);
-			lib_put(thread->dso_data);
-
-			lock_mutex();
-			if (list_empty(&thread_registry))
-				exit_dm_lib();
-			unlock_mutex();
-		}
+		UNLINK_THREAD(thread);
+		LINK(thread, &thread_registry_unused);
 	}
-
+	unlock_mutex();
 
    out:
 	return ret;
@@ -1054,7 +1036,8 @@ static int open_fifos(struct dm_event_fifos *fifos)
  */
 static int client_read(struct dm_event_fifos *fifos, struct dm_event_daemon_message *msg)
 {
-	int bytes = 0, ret = 0;
+	unsigned bytes = 0;
+	int ret = 0;
 	fd_set fds;
 
 	errno = 0;
@@ -1077,7 +1060,8 @@ static int client_read(struct dm_event_fifos *fifos, struct dm_event_daemon_mess
  */
 static int client_write(struct dm_event_fifos *fifos, struct dm_event_daemon_message *msg)
 {
-	int bytes = 0, ret = 0;
+	unsigned bytes = 0;
+	int ret = 0;
 	fd_set fds;
 
 	errno = 0;
@@ -1139,7 +1123,6 @@ static int do_process_request(struct dm_event_daemon_message *msg)
 		stack;
 		ret = -EINVAL;
 	} else {
-log_print("%s: %u \"%s\"\n", __func__, msg->opcode.cmd, message_data.msg->msg);
 		ret = handle_request(msg, &message_data);
 	}
 
@@ -1164,11 +1147,43 @@ static void process_request(struct dm_event_fifos *fifos)
 
 	msg.opcode.status = do_process_request(&msg);
 
-log_print("%s: status: %s\n", __func__, strerror(-msg.opcode.status));
 	if (!client_write(fifos, &msg))
 		stack;
 }
 
+static void cleanup_unused_threads(void)
+{
+	int ret;
+	struct list *l;
+	struct thread_status *thread;
+
+	lock_mutex();
+	while ((l = list_first(&thread_registry_unused))) {
+		thread = list_item(l, struct thread_status);
+		if (thread->processing) {
+			goto out;  /* cleanup on the next round */
+		}
+		list_del(l);
+
+		if (!thread->events) {
+			/* turn codes negative -- should we be returning this? */
+			if ((ret = -terminate_thread(thread)))
+				stack;
+			else {
+				pthread_join(thread->thread, NULL);
+				lib_put(thread->dso_data);
+				free_thread_status(thread);
+			}
+		} else {
+			log_error("thread can't be on unused list unless !thread->events");
+			LINK_THREAD(thread);
+		}
+
+	}
+out:
+	unlock_mutex();
+}
+
 static void sig_alarm(int signum)
 {
 	pthread_testcancel();
@@ -1264,8 +1279,25 @@ void dmeventd(void)
 	 */
 	do {
 		process_request(&fifos);
+		cleanup_unused_threads();
 	} while(!list_empty(&thread_registry));
 
+	/*
+	 * There may still have been some threads that were doing work,
+	 * make sure these are cleaned up
+	 *
+	 * I don't necessarily like the sleep there, but otherwise,
+	 * cleanup_unused_threads could get called many many times.
+	 * It's worth noting that the likelyhood of it being called
+	 * here is slim.
+	 */
+	while(!list_empty(&thread_registry_unused)) {
+		sleep(1);
+		cleanup_unused_threads();
+	}
+
+	exit_dm_lib();
+
 #ifdef MCL_CURRENT
 	munlockall();
 #endif
