@@ -86,8 +86,10 @@ enum {
 	READ_ONLY = 0,
 	COLS_ARG,
 	EXEC_ARG,
+	GID_ARG,
 	MAJOR_ARG,
 	MINOR_ARG,
+	MODE_ARG,
 	NOHEADINGS_ARG,
 	NOLOCKFS_ARG,
 	NOOPENCOUNT_ARG,
@@ -95,6 +97,7 @@ enum {
 	OPTIONS_ARG,
 	TARGET_ARG,
 	TREE_ARG,
+	UID_ARG,
 	UUID_ARG,
 	VERBOSE_ARG,
 	VERSION_ARG,
@@ -390,6 +393,15 @@ static int _create(int argc, char **argv, void *data)
 	if (_switches[MINOR_ARG] && !dm_task_set_minor(dmt, _values[MINOR_ARG]))
 		goto out;
 
+	if (_switches[UID_ARG] && !dm_task_set_uid(dmt, _values[UID_ARG]))
+		goto out;
+
+	if (_switches[GID_ARG] && !dm_task_set_gid(dmt, _values[GID_ARG]))
+		goto out;
+
+	if (_switches[MODE_ARG] && !dm_task_set_mode(dmt, _values[MODE_ARG]))
+		goto out;
+
 	if (_switches[NOOPENCOUNT_ARG] && !dm_task_no_open_count(dmt))
 		goto out;
 
@@ -438,7 +450,8 @@ static int _rename(int argc, char **argv, void *data)
 
 static int _message(int argc, char **argv, void *data)
 {
-	int r = 0, sz = 1, i;
+	int r = 0, i;
+	size_t sz = 1;
 	struct dm_task *dmt;
 	char *str;
 
@@ -455,7 +468,7 @@ static int _message(int argc, char **argv, void *data)
 		argv++;
 	}
 
-	if (!dm_task_set_sector(dmt, atoll(argv[1])))
+	if (!dm_task_set_sector(dmt, (uint64_t) atoll(argv[1])))
 		goto out;
 
 	argc -= 2;
@@ -502,7 +515,7 @@ static int _version(int argc, char **argv, void *data)
 	if (dm_get_library_version(version, sizeof(version)))
 		printf("Library version:   %s\n", version);
 
-	if (dm_driver_version(version, sizeof(version)))
+	if (!dm_driver_version(version, sizeof(version)))
 		return 0;
 
 	printf("Driver version:    %s\n", version);
@@ -580,7 +593,7 @@ static int _wait(int argc, char **argv, void *data)
 	}
 
 	return _simple(DM_DEVICE_WAITEVENT, name,
-		       (argc > 1) ? atoi(argv[argc - 1]) : 0, 1);
+		       (argc > 1) ? (uint32_t) atoi(argv[argc - 1]) : 0, 1);
 }
 
 static int _process_all(int argc, char **argv,
@@ -622,7 +635,7 @@ static int _process_all(int argc, char **argv,
 	return r;
 }
 
-static void _display_dev(struct dm_task *dmt, char *name)
+static void _display_dev(struct dm_task *dmt, const char *name)
 {
 	struct dm_info info;
 
@@ -635,7 +648,7 @@ static int _mknodes(int argc, char **argv, void *data)
 	return dm_mknodes(argc > 1 ? argv[1] : NULL);
 }
 
-static int _exec_command(char *name)
+static int _exec_command(const char *name)
 {
 	int n;
 	static char path[PATH_MAX];
@@ -651,7 +664,7 @@ static int _exec_command(char *name)
 		return 0;
 
 	n = snprintf(path, sizeof(path), "%s/%s", dm_dir(), name);
-	if (n < 0 || n > sizeof(path) - 1)
+	if (n < 0 || n > (int) sizeof(path) - 1)
 		return 0;
 
 	if (!argc) {
@@ -704,7 +717,7 @@ static int _status(int argc, char **argv, void *data)
 	char *params;
 	int cmd;
 	struct dm_names *names = (struct dm_names *) data;
-	char *name = NULL;
+	const char *name = NULL;
 	int matched = 0;
 	int ls_only = 0;
 	struct dm_info info;
@@ -742,7 +755,7 @@ static int _status(int argc, char **argv, void *data)
 		goto out;
 
 	if (!name)
-		name = (char *) dm_task_get_name(dmt);
+		name = dm_task_get_name(dmt);
 
 	/* Fetch targets and print 'em */
 	do {
@@ -1292,6 +1305,7 @@ struct command {
 
 static struct command _commands[] = {
 	{"create", "<dev_name> [-j|--major <major> -m|--minor <minor>]\n"
+	  "\t                  [-U|--uid <uid>] [-G|--gid <gid>] [-M|--mode <octal_mode>]\n"
 	  "\t                  [-u|uuid <uuid>] [--notable] [<table_file>]",
 	 1, 2, _create},
 	{"remove", "<device>", 0, 1, _remove},
@@ -1418,8 +1432,10 @@ static int _process_switches(int *argc, char ***argv)
 		{"readonly", 0, &ind, READ_ONLY},
 		{"columns", 0, &ind, COLS_ARG},
 		{"exec", 1, &ind, EXEC_ARG},
+		{"gid", 1, &ind, GID_ARG},
 		{"major", 1, &ind, MAJOR_ARG},
 		{"minor", 1, &ind, MINOR_ARG},
+		{"mode", 1, &ind, MODE_ARG},
 		{"noheadings", 0, &ind, NOHEADINGS_ARG},
 		{"nolockfs", 0, &ind, NOLOCKFS_ARG},
 		{"noopencount", 0, &ind, NOOPENCOUNT_ARG},
@@ -1427,6 +1443,7 @@ static int _process_switches(int *argc, char ***argv)
 		{"options", 1, &ind, OPTIONS_ARG},
 		{"target", 1, &ind, TARGET_ARG},
 		{"tree", 0, &ind, TREE_ARG},
+		{"uid", 1, &ind, UID_ARG},
 		{"uuid", 1, &ind, UUID_ARG},
 		{"verbose", 1, &ind, VERBOSE_ARG},
 		{"version", 0, &ind, VERSION_ARG},
@@ -1478,7 +1495,7 @@ static int _process_switches(int *argc, char ***argv)
 
 	optarg = 0;
 	optind = OPTIND_INIT;
-	while ((ind = -1, c = GETOPTLONG_FN(*argc, *argv, "cCj:m:no:ru:v",
+	while ((ind = -1, c = GETOPTLONG_FN(*argc, *argv, "cCGj:m:Mno:ru:Uv",
 					    long_options, NULL)) != -1) {
 		if (c == 'c' || c == 'C' || ind == COLS_ARG)
 			_switches[COLS_ARG]++;
@@ -1504,6 +1521,19 @@ static int _process_switches(int *argc, char ***argv)
 			_switches[UUID_ARG]++;
 			_uuid = optarg;
 		}
+		if (c == 'G' || ind == GID_ARG) {
+			_switches[GID_ARG]++;
+			_values[GID_ARG] = atoi(optarg);
+		}
+		if (c == 'U' || ind == UID_ARG) {
+			_switches[UID_ARG]++;
+			_values[UID_ARG] = atoi(optarg);
+		}
+		if (c == 'M' || ind == MODE_ARG) {
+			_switches[MODE_ARG]++;
+			/* FIXME Accept modes as per chmod */
+			_values[MODE_ARG] = (int) strtol(optarg, NULL, 8);
+		}
 		if ((ind == EXEC_ARG)) {
 			_switches[EXEC_ARG]++;
 			_command = optarg;
