@@ -61,14 +61,14 @@ static void _default_log(int level, const char *file, int line,
 		fprintf(stdout, "\n");
 }
 
-dm_log_fn _log = _default_log;
+dm_log_fn dm_log = _default_log;
 
 void dm_log_init(dm_log_fn fn)
 {
 	if (fn)
-		_log = fn;
+		dm_log = fn;
 	else
-		_log = _default_log;
+		dm_log = _default_log;
 }
 
 void dm_log_init_verbose(int level)
@@ -96,7 +96,8 @@ struct dm_task *dm_task_create(int type)
 	struct dm_task *dmt = dm_malloc(sizeof(*dmt));
 
 	if (!dmt) {
-		log_error("dm_task_create: malloc(%d) failed", sizeof(*dmt));
+		log_error("dm_task_create: malloc(%" PRIsize_t ") failed",
+			  sizeof(*dmt));
 		return NULL;
 	}
 
@@ -180,6 +181,27 @@ int dm_task_set_minor(struct dm_task *dmt, int minor)
 	return 1;
 }
 
+int dm_task_set_uid(struct dm_task *dmt, uid_t uid)
+{
+	dmt->uid = uid;
+
+	return 1;
+}
+
+int dm_task_set_gid(struct dm_task *dmt, gid_t gid)
+{
+	dmt->gid = gid;
+
+	return 1;
+}
+
+int dm_task_set_mode(struct dm_task *dmt, mode_t mode)
+{
+	dmt->mode = mode;
+
+	return 1;
+}
+
 int dm_task_add_target(struct dm_task *dmt, uint64_t start, uint64_t size,
 		       const char *ttype, const char *params)
 {
